# -*- coding: utf-8 -*-
#
# DVD controller Plugin by gutemine
#
# Version 0.7
#

from Plugins.Plugin import PluginDescriptor
from Components.config import config, ConfigSubsection, ConfigEnableDisable, ConfigInteger
from enigma import eTimer
from Screens.Screen import Screen
from Components.ConfigList import ConfigListScreen
from Screens.Setup import SetupSummary
from Screens.MessageBox import MessageBox
from Components.ActionMap import ActionMap
from Components.config import config, getConfigListEntry
from Components.Label import Label

import sys, os, struct
from fcntl import ioctl
from struct import pack, unpack

dvdcontroller_plugindir="/usr/lib/enigma2/python/Plugins/Extensions/DVDcontroller"
dvdcontroller_period=10000
if os.path.exists("/dev/sr0"):
   dvdcontroller_device="/dev/sr0"
else:
   dvdcontroller_device="/dev/scsi/host1/bus0/target0/lun0/cd"

config.plugins.dvdcontroller = ConfigSubsection()
config.plugins.dvdcontroller.speed = ConfigInteger(default = 2, limits = (0, 10))
config.plugins.dvdcontroller.period = ConfigInteger(default = 5, limits = (0, 10))

def autostart(reason, **kwargs):
    global session
    if kwargs.has_key("session") and reason == 0:
       session = kwargs["session"]
       print "[DVDCONTROLLER] autostart check"
       session.open(StartupDVDController)

def Plugins(**kwargs):
    return [PluginDescriptor(where = [PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc = autostart),
    		PluginDescriptor(name=_("DVD Controller"), description=_("controll your DVD/CD Drive"), where = PluginDescriptor.WHERE_PLUGINMENU, icon="dvdcontroller.png", fnc=main),
		PluginDescriptor(name=_("DVD Controller"), description=_("controll your DVD/CD Drive"), where = PluginDescriptor.WHERE_EXTENSIONSMENU, icon="dvdcontroller.png", fnc=main)]

def main(session, **kwargs):
        print "[DVDCONTROLLER] Open Config Screen"
        session.open(CheckSoftwareUpdateConfiguration)

class CheckSoftwareUpdateConfiguration(Screen, ConfigListScreen):
    skin = """
        <screen position="center,center" size="600,200" title="Configuration for DVD/CD Drive Controller" >
        <widget name="config" position="0,0" size="590,190" scrollbarMode="showOnDemand" />
        <widget name="buttonred" position="10,140" size="100,40" backgroundColor="red" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="buttongreen" position="120,140" size="100,40" backgroundColor="green" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="buttonyellow" position="230,140" size="100,40" backgroundColor="yellow" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="buttonblue" position="340,140" size="100,40" backgroundColor="blue" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        </screen>"""

    def __init__(self, session, args = 0):
	Screen.__init__(self, session)
       	self.list = []
        self.list.append(getConfigListEntry(_("DVD Speed 0-10 (0=auto)"), config.plugins.dvdcontroller.speed))
        self.list.append(getConfigListEntry(_("Periodic DVD Speed setting 0-10 min (0 only on startup)"), config.plugins.dvdcontroller.period))
        self.onShown.append(self.setWindowTitle)
       	ConfigListScreen.__init__(self, self.list)

        # explizit check on every entry
	self.onChangedEntry = []

       	self["buttonred"] = Label(_("Cancel"))
       	self["buttongreen"] = Label(_("OK"))
       	self["buttonyellow"] = Label(_("Force Eject"))
       	self["buttonblue"] = Label(_("Set Speed"))
        self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
       	{
       		"green": self.save,
        	"red": self.cancel,
        	"yellow": self.dvdeject,
        	"blue": self.dvdspeed,
            	"save": self.save,
            	"cancel": self.cancel,
            	"ok": self.save,
       	})
    def setWindowTitle(self):
        self.setTitle(_("Configuration for DVD/CD Drive Controller"))

    def save(self):
        for x in self["config"].list:
           x[1].save()
        self.close(True)

    def cancel(self):
        for x in self["config"].list:
           x[1].cancel()
        self.close(False)

    def dvdspeed(self):
        if os.path.exists(dvdcontroller_device):
           ioctl_flag=int(0x5322)
           try:
              cd = open(dvdcontroller_device)
              ioctl(cd.fileno(), ioctl_flag, int(config.plugins.dvdcontroller.speed.value)) 
              cd.close()
              self.session.open(MessageBox,_("Setting of Speed of DVD/CD succeeded"), type = MessageBox.TYPE_INFO, timeout =  5)		
           except IOError:
              self.session.open(MessageBox,_("Setting of Speed of DVD/CD failed"), type = MessageBox.TYPE_ERROR, timeout =  5)		
        else:
           self.session.open(MessageBox,_("DVD/CD device not found"), type = MessageBox.TYPE_ERROR, timeout =  5)		
        
    def dvdeject(self):
        if os.path.exists(dvdcontroller_device):
           self.session.openWithCallback(self.doeject,MessageBox,_("Front panel open ?\nPress OK to eject!\n\nFrontklappe offen ?\nAuswerfen auf OK !"), type = MessageBox.TYPE_WARNING)		
        else:
           self.session.open(MessageBox,_("DVD/CD device not found"), type = MessageBox.TYPE_ERROR, timeout =  5)		

    def doeject(self,retval):
        try:
           cd = open(dvdcontroller_device)
           # Stop the cdrom drive
           ioctl_flag=int(0x5307)
           ioctl(cd.fileno(), ioctl_flag) 
           # hard-reset the drive
#          ioctl_flag=int(0x5312)
#          ioctl(cd.fileno(), ioctl_flag) 
           # lock or unlock door
           ioctl_flag=int(0x5329)
           ioctl(cd.fileno(), ioctl_flag) 
           # Ejects the cdrom media
           ioctl_flag=int(0x5309)
           ioctl(cd.fileno(), ioctl_flag) 
           cd.close()
        except IOError:
           self.session.open(MessageBox,_("Eject of DVD/CD failed"), type = MessageBox.TYPE_ERROR, timeout =  5)		

class StartupDVDController(Screen):
    skin = """
        <screen position="center,center" size="400,300" title="Setting DVD Speed" >
        </screen>"""
    def __init__(self,session):
        self.skin = StartupDVDController.skin
	self.session = session
	Screen.__init__(self,session)

        print "[DVDCONTROLLER] doing check"
        self.TimerStartupDVDController = eTimer()
        self.TimerStartupDVDController.stop()
        self.TimerStartupDVDController.timeout.get().append(self.SetDVDControllerSpeed)
        # 10 sec after enigma2 startup we set the speed once
        self.TimerStartupDVDController.start(10000,True) 

    def SetDVDControllerSpeed(self):
        self.TimerStartupDVDController.stop()
        if os.path.exists(dvdcontroller_device):
           ioctl_flag=int(0x5322)
           try:
              cd = open(dvdcontroller_device)
              ioctl(cd.fileno(), ioctl_flag, int(config.plugins.dvdcontroller.speed.value)) 
              cd.close()
           except IOError:
              pass
        # if regular speed setting is enabled  re-shedule
        if config.plugins.dvdcontroller.period.value > 0:
           self.TimerStartupDVDController.start(config.plugins.dvdcontroller.period.value*60000,True) 

